# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "ResponseCreateParams",
    "Input",
    "InputResponseInputMessage",
    "InputResponseInputMessageContent",
    "InputResponseInputMessageContentResponseInputText",
    "InputResponseInputMessageContentResponseInputFile",
    "InputResponseOutputMessage",
    "InputResponseOutputMessageContent",
    "InputResponseOutputMessageContentResponseOutputText",
    "InputResponseOutputMessageContentResponseOutputTextAnnotation",
    "InputResponseOutputMessageContentResponseOutputRefusal",
    "InputResponseReasoning",
    "InputResponseReasoningSummary",
]


class ResponseCreateParams(TypedDict, total=False):
    input: Required[Iterable[Input]]

    include: List[Literal["reasoning.encrypted_content"]]

    model: Literal["witan-alfred", "witan-alfred-mini", "witan-edward", "witan-edward-mini"]

    stream: bool


class InputResponseInputMessageContentResponseInputText(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["input_text"]]


class InputResponseInputMessageContentResponseInputFile(TypedDict, total=False):
    filename: Required[str]

    type: Required[Literal["input_file"]]

    file_data: str

    file_id: str


InputResponseInputMessageContent: TypeAlias = Union[
    InputResponseInputMessageContentResponseInputText, InputResponseInputMessageContentResponseInputFile
]


class InputResponseInputMessage(TypedDict, total=False):
    content: Required[Iterable[InputResponseInputMessageContent]]

    role: Required[Literal["user", "system", "developer"]]

    type: Literal["message"]


class InputResponseOutputMessageContentResponseOutputTextAnnotation(TypedDict, total=False):
    type: Required[str]


class InputResponseOutputMessageContentResponseOutputText(TypedDict, total=False):
    annotations: Required[Iterable[InputResponseOutputMessageContentResponseOutputTextAnnotation]]

    text: Required[str]

    type: Required[Literal["output_text"]]


class InputResponseOutputMessageContentResponseOutputRefusal(TypedDict, total=False):
    refusal: Required[str]

    type: Required[Literal["refusal"]]


InputResponseOutputMessageContent: TypeAlias = Union[
    InputResponseOutputMessageContentResponseOutputText, InputResponseOutputMessageContentResponseOutputRefusal
]


class InputResponseOutputMessage(TypedDict, total=False):
    id: Required[str]

    content: Required[Iterable[InputResponseOutputMessageContent]]

    role: Required[Literal["assistant"]]

    status: Required[Literal["in_progress", "completed", "incomplete"]]

    type: Required[Literal["message"]]


class InputResponseReasoningSummary(TypedDict, total=False):
    text: Required[str]

    type: Required[Literal["summary_text"]]


class InputResponseReasoning(TypedDict, total=False):
    id: Required[str]

    summary: Required[Iterable[InputResponseReasoningSummary]]

    type: Required[Literal["reasoning"]]

    encrypted_content: str

    status: Literal["in_progress", "completed", "incomplete"]


Input: TypeAlias = Union[InputResponseInputMessage, InputResponseOutputMessage, InputResponseReasoning]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .file_list_params import FileListParams as FileListParams
from .file_list_response import FileListResponse as FileListResponse
from .file_upload_params import FileUploadParams as FileUploadParams
from .file_delete_response import FileDeleteResponse as FileDeleteResponse
from .file_upload_response import FileUploadResponse as FileUploadResponse
from .file_retrieve_response import FileRetrieveResponse as FileRetrieveResponse
from .response_create_params import ResponseCreateParams as ResponseCreateParams
from .response_create_response import ResponseCreateResponse as ResponseCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from .._models import BaseModel

__all__ = [
    "ResponseCreateResponse",
    "Error",
    "Output",
    "OutputResponseOutputMessage",
    "OutputResponseOutputMessageContent",
    "OutputResponseOutputMessageContentResponseOutputText",
    "OutputResponseOutputMessageContentResponseOutputTextAnnotation",
    "OutputResponseOutputMessageContentResponseOutputRefusal",
    "OutputResponseReasoning",
    "OutputResponseReasoningSummary",
    "Tool",
]


class Error(BaseModel):
    code: Literal["server_error"]

    message: str


class OutputResponseOutputMessageContentResponseOutputTextAnnotation(BaseModel):
    type: str


class OutputResponseOutputMessageContentResponseOutputText(BaseModel):
    annotations: List[OutputResponseOutputMessageContentResponseOutputTextAnnotation]

    text: str

    type: Literal["output_text"]


class OutputResponseOutputMessageContentResponseOutputRefusal(BaseModel):
    refusal: str

    type: Literal["refusal"]


OutputResponseOutputMessageContent: TypeAlias = Union[
    OutputResponseOutputMessageContentResponseOutputText, OutputResponseOutputMessageContentResponseOutputRefusal
]


class OutputResponseOutputMessage(BaseModel):
    id: str

    content: List[OutputResponseOutputMessageContent]

    role: Literal["assistant"]

    status: Literal["in_progress", "completed", "incomplete"]

    type: Literal["message"]


class OutputResponseReasoningSummary(BaseModel):
    text: str

    type: Literal["summary_text"]


class OutputResponseReasoning(BaseModel):
    id: str

    summary: List[OutputResponseReasoningSummary]

    type: Literal["reasoning"]

    encrypted_content: Optional[str] = None

    status: Optional[Literal["in_progress", "completed", "incomplete"]] = None


Output: TypeAlias = Union[OutputResponseOutputMessage, OutputResponseReasoning]


class Tool(BaseModel):
    type: str


class ResponseCreateResponse(BaseModel):
    id: str

    created_at: int

    error: Optional[Error] = None

    incomplete_details: None = None

    instructions: None = None

    metadata: None = None

    model: Literal["witan-alfred", "witan-alfred-mini", "witan-edward", "witan-edward-mini"]

    object: Literal["response"]

    output: List[Output]

    output_text: str

    parallel_tool_calls: bool

    status: Literal["completed", "in_progress", "failed"]

    temperature: None = None

    tool_choice: Literal["none"]

    tools: List[Tool]

    top_p: None = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["FileListResponse"]


class FileListResponse(BaseModel):
    id: str

    bytes: float

    created_at: float

    filename: str

    object: Literal["file"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .files import (
    FilesResource,
    AsyncFilesResource,
    FilesResourceWithRawResponse,
    AsyncFilesResourceWithRawResponse,
    FilesResourceWithStreamingResponse,
    AsyncFilesResourceWithStreamingResponse,
)
from .responses import (
    ResponsesResource,
    AsyncResponsesResource,
    ResponsesResourceWithRawResponse,
    AsyncResponsesResourceWithRawResponse,
    ResponsesResourceWithStreamingResponse,
    AsyncResponsesResourceWithStreamingResponse,
)

__all__ = [
    "FilesResource",
    "AsyncFilesResource",
    "FilesResourceWithRawResponse",
    "AsyncFilesResourceWithRawResponse",
    "FilesResourceWithStreamingResponse",
    "AsyncFilesResourceWithStreamingResponse",
    "ResponsesResource",
    "AsyncResponsesResource",
    "ResponsesResourceWithRawResponse",
    "AsyncResponsesResourceWithRawResponse",
    "ResponsesResourceWithStreamingResponse",
    "AsyncResponsesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from witan import Witan, AsyncWitan
from tests.utils import assert_matches_type
from witan.types import ResponseCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestResponses:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Witan) -> None:
        response = client.responses.create(
            input=[
                {
                    "content": [
                        {
                            "text": "x",
                            "type": "input_text",
                        }
                    ],
                    "role": "user",
                }
            ],
        )
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Witan) -> None:
        response = client.responses.create(
            input=[
                {
                    "content": [
                        {
                            "text": "x",
                            "type": "input_text",
                        }
                    ],
                    "role": "user",
                    "type": "message",
                }
            ],
            include=["reasoning.encrypted_content"],
            model="witan-alfred",
            stream=True,
        )
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Witan) -> None:
        http_response = client.responses.with_raw_response.create(
            input=[
                {
                    "content": [
                        {
                            "text": "x",
                            "type": "input_text",
                        }
                    ],
                    "role": "user",
                }
            ],
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Witan) -> None:
        with client.responses.with_streaming_response.create(
            input=[
                {
                    "content": [
                        {
                            "text": "x",
                            "type": "input_text",
                        }
                    ],
                    "role": "user",
                }
            ],
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert_matches_type(ResponseCreateResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True


class TestAsyncResponses:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncWitan) -> None:
        response = await async_client.responses.create(
            input=[
                {
                    "content": [
                        {
                            "text": "x",
                            "type": "input_text",
                        }
                    ],
                    "role": "user",
                }
            ],
        )
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncWitan) -> None:
        response = await async_client.responses.create(
            input=[
                {
                    "content": [
                        {
                            "text": "x",
                            "type": "input_text",
                        }
                    ],
                    "role": "user",
                    "type": "message",
                }
            ],
            include=["reasoning.encrypted_content"],
            model="witan-alfred",
            stream=True,
        )
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncWitan) -> None:
        http_response = await async_client.responses.with_raw_response.create(
            input=[
                {
                    "content": [
                        {
                            "text": "x",
                            "type": "input_text",
                        }
                    ],
                    "role": "user",
                }
            ],
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = await http_response.parse()
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncWitan) -> None:
        async with async_client.responses.with_streaming_response.create(
            input=[
                {
                    "content": [
                        {
                            "text": "x",
                            "type": "input_text",
                        }
                    ],
                    "role": "user",
                }
            ],
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert_matches_type(ResponseCreateResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ..types import response_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.response_create_response import ResponseCreateResponse

__all__ = ["ResponsesResource", "AsyncResponsesResource"]


class ResponsesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ResponsesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/witanlabs/witan-python#accessing-raw-response-data-eg-headers
        """
        return ResponsesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResponsesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/witanlabs/witan-python#with_streaming_response
        """
        return ResponsesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        input: Iterable[response_create_params.Input],
        include: List[Literal["reasoning.encrypted_content"]] | Omit = omit,
        model: Literal["witan-alfred", "witan-alfred-mini", "witan-edward", "witan-edward-mini"] | Omit = omit,
        stream: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResponseCreateResponse:
        """
        Create a model response

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v0/responses",
            body=maybe_transform(
                {
                    "input": input,
                    "include": include,
                    "model": model,
                    "stream": stream,
                },
                response_create_params.ResponseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ResponseCreateResponse,
        )


class AsyncResponsesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncResponsesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/witanlabs/witan-python#accessing-raw-response-data-eg-headers
        """
        return AsyncResponsesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResponsesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/witanlabs/witan-python#with_streaming_response
        """
        return AsyncResponsesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        input: Iterable[response_create_params.Input],
        include: List[Literal["reasoning.encrypted_content"]] | Omit = omit,
        model: Literal["witan-alfred", "witan-alfred-mini", "witan-edward", "witan-edward-mini"] | Omit = omit,
        stream: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResponseCreateResponse:
        """
        Create a model response

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v0/responses",
            body=await async_maybe_transform(
                {
                    "input": input,
                    "include": include,
                    "model": model,
                    "stream": stream,
                },
                response_create_params.ResponseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ResponseCreateResponse,
        )


class ResponsesResourceWithRawResponse:
    def __init__(self, responses: ResponsesResource) -> None:
        self._responses = responses

        self.create = to_raw_response_wrapper(
            responses.create,
        )


class AsyncResponsesResourceWithRawResponse:
    def __init__(self, responses: AsyncResponsesResource) -> None:
        self._responses = responses

        self.create = async_to_raw_response_wrapper(
            responses.create,
        )


class ResponsesResourceWithStreamingResponse:
    def __init__(self, responses: ResponsesResource) -> None:
        self._responses = responses

        self.create = to_streamed_response_wrapper(
            responses.create,
        )


class AsyncResponsesResourceWithStreamingResponse:
    def __init__(self, responses: AsyncResponsesResource) -> None:
        self._responses = responses

        self.create = async_to_streamed_response_wrapper(
            responses.create,
        )
